@@ -1,4 +1,4 @@
-# $Id: /mirror/gungho/lib/Gungho/Engine/POE.pm 8885 2007-11-10T07:19:07.305088Z lestrrat  $
+# $Id: /mirror/gungho/lib/Gungho/Engine/POE.pm 31130 2007-11-27T01:45:05.109813Z lestrrat  $
 #
 # Copyright (c) 2007 Daisuke Maki <daisuke@endeworks.jp>
 # All rights reserved.
@@ -135,7 +135,7 @@ sub run
                 _start => '_poe_session_start',
                 _stop  => '_poe_session_stop',
                 map { ($_ => "_poe_$_") }
-                    qw(session_loop start_request handle_response got_dns_response)
+                    qw(session_loop start_request handle_response got_dns_response shutdown)
             }
         ]
     );
@@ -143,6 +143,23 @@ sub run
     POE::Kernel->run();
 }
 
+sub stop
+{
+    my ($self, $c) = @_;
+    POE::Kernel->post($self->alias, 'shutdown');
+}
+
+sub _poe_shutdown
+{
+    my ($self, $kernel, $heap) = @_[OBJECT, KERNEL, HEAP];
+    my $clients = $self->clients;
+    foreach my $client (@$clients) {
+        $kernel->post($client, 'shutdown');
+    }
+    my $c = $heap->{CONTEXT};
+    $c->is_running(0);
+}
+
 sub _poe_session_start
 {
     $_[KERNEL]->alias_set( $_[OBJECT]->alias );
@@ -181,13 +198,14 @@ sub _poe_session_loop
     }
     $self->loop_alarm($kernel->alarm_set('session_loop', time() + $delay));
 
-    $c->run_hook('engine.end_loop');
+    $c->notify('engine.end_loop');
 }
 
 sub send_request
 {
     my ($self, $c, $request) = @_;
     POE::Kernel->post($self->alias, 'start_request', $request);
+    return 1;
 }
 
 sub _poe_start_request
@@ -217,7 +235,7 @@ sub _poe_start_request
         return;
     }
 
-    $c->run_hook('engine.send_request', { request => $request });
+    $c->notify('engine.send_request', { request => $request });
 
     if (DEBUG) {
         my $uri = $request->uri->clone;
@@ -277,7 +295,7 @@ sub _poe_handle_response
         }
     }
 
-    $c->run_hook('engine.handle_response', { request => $req, response => $res });
+    $c->notify('engine.handle_response', { request => $req, response => $res });
 
     # Do we support auth challenge ?
     my $code = $c->can('check_authentication_challenge');
@@ -441,6 +459,10 @@ sets up the engine.
 Instantiates a PoCo::Client::HTTP session and a main session that handles the
 main control.
 
+=head2 stop
+
+Shutsdown the engine
+
 =head2 send_request($request)
 
 Sends a request to the http client
@@ -448,14 +470,16 @@ Sends a request to the http client
 =head1 CAVEATS
 
 The POE engine supports multiple values in the user-agent header, but this
-is an exception. To be portable with other engines, and if you are using only
-one user-agent, set it at the top level:
+is an exception that other engines don't support. Please use define your
+agent strings in the top level config:
 
   user_agent: my_user_agent
   engine:
     module: POE
     ...
 
+If you don't do this, components such as RobotRules won't work properly
+
 =head1 TODO
 
 Xango, Gungho's predecessor, tried really hard to overcome one of my pet-peeves
